{************************************************************

      Some console routins I've started to develop for
      myself

      Copyright (C) Roman Kulish <r_kulish@yahoo.com>                                                       

*************************************************************}

unit _console;

interface

uses Windows, SysUtils;

//cursor functions
procedure ShowConsoleCursor(Visible: Boolean);
procedure PutCursorAt(Offset, X: Integer); //Offset - could be negative or positive value

//output functions
procedure CWriteLn(Text: String; Color: Word);
procedure LWrite(Text: String; Color: Word; BreakLine: Byte);
procedure Progress(Text: String; Percent: Byte; Before, After, Current: Char; Color: Word);

//some routins
procedure ClearRow(Offset: Integer); //Offset - could be negative or positive value

{$R _console.res}

implementation

//*** shows console cursor function (true - visible, false - hidden) ***
procedure ShowConsoleCursor(Visible: Boolean);
var
   ccinfo: _CONSOLE_CURSOR_INFO;
begin
     GetConsoleCursorInfo(GetStdHandle(STD_OUTPUT_HANDLE), ccinfo);
     ccinfo.bVisible:= Visible;
     SetConsoleCursorInfo(GetStdHandle(STD_OUTPUT_HANDLE), ccinfo);
end;

//*** puts cursor at CURRENT_ROW + Offset and X column
procedure PutCursorAt(Offset, X: Integer);
var
   coord: _COORD;
   sbinfo: _CONSOLE_SCREEN_BUFFER_INFO;
begin
     GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), sbinfo);
     if X <> -1 then coord.X:= X else
     coord.X:= sbinfo.dwCursorPosition.X;
     coord.Y:= sbinfo.dwCursorPosition.Y + Offset;
     SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), coord);
end;

//*** Writes center and colored text
procedure CWriteLn(Text: String; Color: Word);
var
   coord: _COORD;
   sbinfo: _CONSOLE_SCREEN_BUFFER_INFO;
   tmpColor: Word;
begin
     GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), sbinfo);
     tmpColor:= sbinfo.wAttributes;
     coord.Y:= sbinfo.dwCursorPosition.Y;
     coord.X:= (sbinfo.dwSize.X - Length(Text)) div 2;
     SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), coord);
     SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), Color);
     WriteLn(Text);
     SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), tmpColor);
end;

//*** writes colorde text at the left, if BreakLine = true then
//*** writes text as WriteLn() function else as Write() function
procedure LWrite(Text: String; Color: Word; BreakLine: Byte);
var
   tmpColor: Word;
   sbinfo: _CONSOLE_SCREEN_BUFFER_INFO;
begin
     GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), sbinfo);
     tmpColor:= sbinfo.wAttributes;
     SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), Color);
     if Boolean(BreakLine) then WriteLn(Text) else Write(Text);
     SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), tmpColor);
end;

//*** writes simple progress bar
procedure Progress(Text: String; Percent: Byte; Before, After, Current: Char; Color: Word);
var
   sbinfo: _CONSOLE_SCREEN_BUFFER_INFO;
   tmpColor: Word;
   i, symb: Integer;
begin
     GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), sbinfo);
     tmpColor:= sbinfo.wAttributes;
     SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), Color);
     WriteLn(Text);
     Write('[');
     symb:= ((sbinfo.dwSize.X - 8)*Percent) div 100;
     for i:= 1 to sbinfo.dwSize.X - 7 do
     if i < symb then Write(Before) else
     if i = symb then Write(Current) else
     if i > symb then Write(After);
     Write(Format('] %d%', [Percent]));
     SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), tmpColor);
end;

//*** fast clears specified row 
procedure ClearRow(Offset: Integer);
var
   s: String;
   coord, tmpCoord: _COORD;
   sbinfo: _CONSOLE_SCREEN_BUFFER_INFO;
begin
     GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), sbinfo);
     SetLength(s, sbinfo.dwSize.X);
     FillChar(s, Length(s), #0);
     coord:= sbinfo.dwCursorPosition;
     tmpCoord.X:= 0;
     tmpCoord.Y:= sbinfo.dwCursorPosition.Y + Offset;
     SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), tmpCoord);
     Write(s);
     SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), coord);
end;

end.
