{********************************************************

     RAS WAVE EXTRACTOR

     Copyright (C) Roman Kulish <r_kulish@yahoo.com>

*********************************************************}

program ras_wave_extractor;

{$APPTYPE CONSOLE}

uses
  Windows,
  SysUtils,
  Classes,
  _console in '_console.pas'; //some console functions

var
   out_dir, ras_file, buf, wave_buf, hdr_size: String;
   fs: TFileStream;
   waves_count, total_size, buf_pos: Integer;

begin
{
     creating console, setting it's title and hidding console cursore
}
     AllocConsole();
     SetConsoleTitle('RAS Wave Ripper');
     ShowConsoleCursor(false);

{
     Information at the top of console: app name, may name and e-mail

     CWriteLn() is modified WriteLn() function (see _console.pas)
}

     CWriteLn('RAS Wave Ripper Utility v1.0', FOREGROUND_GREEN);
     CWriteLn('Copyright (C) Roman Kulish, December 2001', FOREGROUND_GREEN);
     CWriteLn('------------ mailto: r_kulish@yahoo.com ------------', FOREGROUND_GREEN);
     WriteLn('');

{
     if no command line parameters (ParaCount = 0) then displaying extra information:

     usage,
     error codes,
     additional information

     LWrite is modified Write() function (see _console.pas)
}

     if ParamCount = 0 then
     begin
          WriteLn('USAGE: ras_wave_extractor.exe [ras_file] [directory_for_waves]');
          WriteLn('');
          WriteLn('EXAMPLE: ras_wave_extractor.exe "c:\Games\Max Payne\x_music.ras" c:\music\');
          WriteLn('');
          WriteLn('');
          CWriteLn('<******************** Error codes *******************>', FOREGROUND_GREEN);
          WriteLn('');
          LWrite('01: ', FOREGROUND_RED or FOREGROUND_GREEN or FOREGROUND_INTENSITY, 0);
          WriteLn('Specified file was not recognized by extractor as valid RAS file.');
          LWrite('02: ', FOREGROUND_RED or FOREGROUND_GREEN or FOREGROUND_INTENSITY, 0);
          WriteLn('File is not found by program.');
          LWrite('03: ', FOREGROUND_RED or FOREGROUND_GREEN or FOREGROUND_INTENSITY, 0);
          WriteLn('Directory cannot be created. Disk may be full or write protected.');
          LWrite('xx: ', FOREGROUND_RED or FOREGROUND_GREEN or FOREGROUND_INTENSITY, 0);
          WriteLn('Other damned Delphi errors.');
          WriteLn('');
          WriteLn('');
          CWriteLn('<***************** Some information *****************>', FOREGROUND_GREEN);
          WriteLn('');
          WriteLn('   This program may be used freely, and you are welcome to redistribute it');
          WriteLn('   under certain conditions.');
          WriteLn('');
          WriteLn('   The author will not make available Technical Support for this Software.');
          WriteLn('   The author may, from time to time, revise or update the Software.');
          WriteLn('');
          WriteLn('   RAS Wave Ripper comes with ABSOLUTELY NO WARRANTY.');
          WriteLn('');
          WriteLn('   Source codes may be sent on demand.');
          WriteLn('');
          LWrite('   Roman Kulish', FOREGROUND_RED or FOREGROUND_BLUE, 1);
          LWrite('   r_kulish@yahoo.com', FOREGROUND_RED or FOREGROUND_BLUE, 1);
          FreeConsole();
          Exit;
     end else

{
     if there any parameters in command line...

     first parameter should be path to .RAS file - ras_file:= Trim(ParamStr(1));
     second parameter is the output directory - out_dir:= Trim(ParamStr(2));

     when parameters were parsed, I do several check procedures...

     if ... <> ... then //something wrong
     begin
          //writting error number
          //writting error explanation
          //freeing console
          //exit procedure
     end;
}

     begin
          ras_file:= Trim(ParamStr(1));
          out_dir:= Trim(ParamStr(2));
          //if file extension is not .ras
          if ExtractFileExt(ras_file) <> '.ras' then
          begin
               LWrite('Error (01):', FOREGROUND_RED or FOREGROUND_INTENSITY, 1);
               LWrite(Format(#9'File is not valid RAS file! - %s', [ras_file]), FOREGROUND_RED or FOREGROUND_INTENSITY, 1);
               FreeConsole();
               Exit;
          end;
          //if file does not exists
          if not FileExists(ras_file) then
          begin
               LWrite('Error (02):', FOREGROUND_RED or FOREGROUND_INTENSITY, 1);
               LWrite(Format(#9'File is not found. - %s', [ras_file]), FOREGROUND_RED or FOREGROUND_INTENSITY, 1);
               FreeConsole();
               Exit;
          end;
          //if output directory is empty, I set current directory as output directory
          if out_dir = '' then out_dir:= ExtractFilePath(ParamStr(3)) else
          //else if directory does not exists, I am trying to create it
          if not DirectoryExists(out_dir) then if not CreateDir(out_dir) then
          begin
               LWrite('Error (03):', FOREGROUND_RED or FOREGROUND_INTENSITY, 1);
               LWrite(Format(#9'Cannot create output directory. - %s', [out_dir]), FOREGROUND_RED or FOREGROUND_INTENSITY, 1);
               FreeConsole();
               Exit;
          end;
{
          Now it time to load archive in variable
          String variable mostly is stored in swap file, so don't pay attention to RAM size
}
          Write(Format('Reading archive %s... ', [ras_file]));
          fs:= TFileStream.Create(ras_file, fmOpenRead);
          SetLength(buf, fs.Size); //always do this!!!
          //to read file content you shopuld provide first element in array! like this: buf[1]
          fs.ReadBuffer(buf[1], fs.Size);
          fs.Free;
          WriteLn(Format('Done, loaded %d kb.', [Length(buf) div 1024]));
          WriteLn('Scanning (this may take up to 5 minutes depending to file size)...');
          WriteLn('');
          //reseting all counters
          waves_count:= 0;
          total_size:= 0;
          buf_pos:= 0;
{
          all functions is implemented inside

          repeat
          ...
          until (position_in_buffer) >= buffer_size

          now application will check symbol-by-symbol for "RIFF....WAVE" string
          if such string is found applcation will calculate wave fle size

          wave file header is written like this

          first 4 bytes are 'RIFF'
          second 4 bytes is file size as WORD
          third 4 bytes is 'WAVE' (audio) or 'AVI ' (video) or etc.

    (TEXT)00 'R' 'I' 'F' 'F'
     (HEX)01 '20' '03' '65' '2A'
    (TEXT)03 'W' 'A' 'V' 'E'

          we must read 4, 5, 6 and 7 symbol after 'RIFF' header in reverse order,
          get their codes in ANSI, convert integers to HEX string, combine string and
          convert it back to Integer

          wave_size:= StrToInt('$' + IntToHEX(ord(buf[buf_pos + 7]), 2) + IntToHEX(ord(buf[buf_pos + 6]), 2) +
          IntToHEX(ord(buf[buf_pos + 5]), 2) + IntToHEX(ord(buf[buf_pos + 4]), 2));
          wave_size:= wave_size + 8; //!!!!! always add 8.

          then I copy wave_size number of character from buf to temporal buffer and save it as .wav file
          and increment buffer position, buf_pos:= buf_pos + (wave_size - 1)
}
          repeat
                //don't make any progress information!!! this will really slow done searching process!
                SetConsoleTitle(PChar(Format('RAS Wave Ripper - Searching [position %d from %d]', [buf_pos, Length(buf)])));
                if (copy(buf, buf_pos, 4) = 'RIFF') and (copy(buf, buf_pos + 8, 4) = 'WAVE') then
                begin
                     inc(waves_count);
                     Write(Format('Writing wave file to %s%s... ', [out_dir, Format('wave%d.wav', [waves_count])]));
                     hdr_size:= IntToHEX(ord(buf[buf_pos + 7]), 2) + IntToHEX(ord(buf[buf_pos + 6]), 2) +
                     IntToHEX(ord(buf[buf_pos + 5]), 2) + IntToHEX(ord(buf[buf_pos + 4]), 2);
                     wave_buf:= copy(buf, buf_pos, StrToInt('$' + hdr_size) + 8);
                     fs:= TFileStream.Create(out_dir + Format('wave%d.wav', [waves_count]), fmCreate or fmOpenWrite);
                     fs.WriteBuffer(wave_buf[1], Length(wave_buf));
                     fs.Free;
                     Inc(buf_pos, Length(wave_buf) - 2);
                     Inc(total_size, Length(wave_buf));
                     WriteLn(Format('Done, %d kb.', [Length(wave_buf) div 1024]));
                     wave_buf:= '';
                end;
                Inc(buf_pos);
          until buf_pos >= Length(buf);
{
          ...now I should free memory by clearing buffer
          write information about how many files app had extracted and total size in kb
}
          buf:= '';
          WriteLn('');
          CWriteLn(Format('Total %d files extracted. About %d kb total size.', [waves_count, total_size div 1024]), FOREGROUND_GREEN);
          WriteLn('');
     end;
     //freeing console
     FreeConsole();
end.
